/*
    Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifdef HAVE_CONFIG_H
#include "autoconfig.h"
#endif

#include <stdio.h>
#include <sstream>

#include "util.h"
#include "json_web_handler.h"

namespace dSA
{

JSONWebHandler::JSONWebHandler(struct mg_connection* connection,
                               const struct mg_request_info* request_info) :
                               WebHandler(connection, request_info) {}

void JSONWebHandler::failure(const std::string message)
{
    std::ostringstream result;
    result << webutil::render_http_header(500, "application/json");
    json_object *ret = json_object_new_object();
    json_object_object_add(ret, "success", json_object_new_boolean(false));
    json_object_object_add(ret, "message",
                json_object_new_string(message.c_str()));
    result << json_object_to_json_string(ret);
    mg_write(m_connection, result.str().c_str(),
             result.str().length());
    json_object_put(ret);
}

std::string JSONWebHandler::http_ok()
{
    return webutil::render_http_header(200, "application/json");
}

json_object* JSONWebHandler::json_ok()
{
    json_object *ret = json_object_new_object();
    json_object_object_add(ret, "success", json_object_new_boolean(true));
    json_object_object_add(ret, "message", json_object_new_string("OK"));
    return ret;
}

} // namespace

