/*
    Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/

#ifdef HAVE_CONFIG_H
    #include "autoconfig.h"
#endif

#include <digitalSTROM/dsm-api-v2/dsm-api.h>
#include <errno.h>

#include "storage.h"
#include "exceptions.h"
#include "database.h"
#include "util.h"

namespace dSA
{

Storage* Storage::m_instance = NULL;

Storage* Storage::getInstance()
{
    if (m_instance == NULL)
    {
        m_instance = new Storage();
        if (m_instance == NULL)
        {
            throw _Exception("Could not create Storage class");
        }
    }

    return m_instance;
}

Storage::Storage() {}

boost::shared_ptr<dSM> Storage::getDSM(dsuid_t dsuid) const
{
    return Database::getInstance()->loadDSM(dsuid);
}

boost::shared_ptr<std::list<dsuid_t> > Storage::getDSMList() const
{
    return Database::getInstance()->getDSMdSUIDs();
}

boost::shared_ptr<std::list<boost::shared_ptr<Device> > >
                                Storage::getDevices(dsuid_t dsm_dsuid,
                                                    bool onlyActive) const
{
    return Database::getInstance()->loadDevices(dsm_dsuid, onlyActive);
}

boost::shared_ptr<std::list<boost::shared_ptr<Device> > >
                                Storage::getDevices(bool onlyActive) const
{
    return Database::getInstance()->loadDevices(onlyActive);
}

boost::shared_ptr<std::list<boost::shared_ptr<Device> > >
         Storage::getDeviceList(dsuid_t dsuid, bool onlyActive) const
{
    return Database::getInstance()->loadDeviceList(dsuid, onlyActive);
}

boost::shared_ptr<std::list<boost::shared_ptr<Device> > >
              Storage::getDeviceList(dsuid_t dsm_dsuid, uint16_t device_id) const
{
    return Database::getInstance()->loadDeviceListM(dsm_dsuid, device_id);
}

void Storage::addDSM(boost::shared_ptr<dSM> dsm)
{
    Database::getInstance()->saveDSM(dsm);
}

boost::shared_ptr<const HWBase> Storage::get(dsuid_t dsuid) const
{
    return Database::getInstance()->loadDevice(dsuid);
}

boost::shared_ptr<Device> Storage::getDevice(dsuid_t dsuid) const
{
    return Database::getInstance()->loadDevice(dsuid);
}

boost::shared_ptr<Device> Storage::getDevice(dsuid_t dsuid,
                                                   dsuid_t dsm_dsuid) const
{
    return Database::getInstance()->loadDevice(dsuid, dsm_dsuid);
}

boost::shared_ptr<Device> Storage::getDevice(uint16_t device_id,
                                                   dsuid_t dsm_dsuid) const
{
    return Database::getInstance()->loadDevice(dsm_dsuid, device_id);
}

void Storage::addDevice(boost::shared_ptr<Device> device)
{
    Database::getInstance()->saveDevice(device);
}

void Storage::removeDevice(dsuid_t dsuid, dsuid_t dsm_dsuid)
{
    Database::getInstance()->deleteDevice(dsuid, dsm_dsuid);
}

void Storage::removeDevice(dsuid_t dsuid)
{
    Database::getInstance()->deleteDevice(dsuid);
}

void Storage::removeDSM(dsuid_t dsuid)
{
    Database::getInstance()->deleteDSM(dsuid);
}

} // namespace
